package com.jboss.test;

import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The Class StoreSessionFilter.
 */
public class StoreSessionFilter implements Filter {

    public static final ConcurrentMap<String, HttpSession> SESSIONS = new ConcurrentHashMap<String, HttpSession>();

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(StoreSessionFilter.class);

    /* (non-Javadoc)
     * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
     */
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
    }

    /* (non-Javadoc)
     * @see javax.servlet.Filter#destroy()
     */
    @Override
    public void destroy() {
    }

    /* (non-Javadoc)
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
     */
    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {
        final HttpServletRequest request = (HttpServletRequest) servletRequest;

        final String sessionId = request.getParameter("logout");
        if (sessionId != null && !sessionId.isEmpty()) {
            LOGGER.info(String.format("===================== Invalidating session: [%s]================", sessionId));

            final HttpSession session = SESSIONS.remove(sessionId);
            if (session != null) {
                try {
                    session.invalidate(); // invalidate the valid session
                    LOGGER.info(String.format(
                            "===================== Invalidate session [%s] successfully ================", sessionId));
                } catch (final IllegalStateException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        } else {
            final HttpSession session = request.getSession(true);

            LOGGER.info(String.format("Storing session for [%s]", session.getId()));
            SESSIONS.put(session.getId(), session);
        }

        filterChain.doFilter(servletRequest, servletResponse);
    }
}
